"""Taxonomy trees for quality control."""

from senselab.audio.tasks.quality_control.taxonomies.bioacoustic import (
    BIOACOUSTIC_ACTIVITY_TAXONOMY,
)
from senselab.audio.tasks.quality_control.taxonomies.bridge2ai_voice import (
    BRIDGE2AI_VOICE_TAXONOMY,
)

__all__ = [
    "BIOACOUSTIC_ACTIVITY_TAXONOMY",
    "BRIDGE2AI_VOICE_TAXONOMY",
]
